/*
 * hardtimes.c
 *
 * This is the actual conversion routines.
 * 
 * Copyright (c) 1993 Bertil K Jonell 
 *
 */

#include<stdio.h>
#include<strings.h>
#include<ctype.h>
#include"include.h"
#include"data.h"

extern struct system sector[][40];
extern struct system newsector[][40];

extern char *port_types[];
extern char *base_types[];
extern char *zone_types[];

FILE *fr;

void hard_times()
{
  int i, j;
  static void process();

  for(i=0; i < 32; i++) {
    for(j=0; j < 40; j++) {
      newsector[i][j] = sector[i][j];
      if (newsector[i][j].presence) {
	process(i,j);
      }
    }
  }
  fclose(fr);
}


void open_report(file)
char file[];
{
  char report_file[MAXSTR];

  strncpy(report_file, file, strlen(file) - 3);
  report_file[strlen(file)-3] = (char)NULL;
  strcat(report_file, "rep");

  if ((fr = fopen(report_file, "w")) == NULL) {
    fprintf(stderr, "I can't open %s\n", report_file);
    exit(1);
  }

  fprintf(stderr, "%s\n", report_file);
}


static void process(i, j)
int i, j;
{
  int dm = -1;
  int dm2 = 0;
  int biodam = 0;
  int popred = 0;
  int techred = 0;
  int xeno = 0;
  int xeno2 = 0;
  int isol = 0;
  int pop = 0;
  int popmul = 0;
  int port = 0;
  int gov = 0;
  int maxpop = 0;
  int maxpopmul = 0;
  int growth = 0;
  int envdm = 0;
  int annihilated = FALSE;
  int lf = FALSE;
  int balk = FALSE;
  int ted = FALSE;
  int hit = FALSE;
  int industrial = FALSE;
  int droyne = 0;
  int chirpers = 0;
  char s[MAXSTR];
  char e[MAXSTR];
  struct system oldsys = sector[i][j];
  struct system newsys = newsector[i][j];

  static int min(), max(), change(), fluid();
  static float isolated();
  char n2h();
  void rem_trail_sp();

  fprintf(fr, "%s (%02d%02d %c%c%c%c%c%c%c-%c%s)\n", oldsys.name, i+1, j+1,
    *port_types[oldsys.port],
    n2h(oldsys.size),
    n2h(oldsys.atm),
    n2h(oldsys.hyd),
    n2h(oldsys.pop),
    n2h(oldsys.gov),
    n2h(oldsys.law),
    n2h(oldsys.tech),
    zone_types[oldsys.zone]);

  /* Bisphere Damage */

  if((dm += oldsys.war) > 0) {
    if(oldsys.port == 0) dm++;
    if(oldsys.pop >= 9) dm++;

    if(oldsys.war > 3) dm2 = 2;
   
    biodam = ((RAND(6) + RAND(6) + dm) >= 14) ? (RAND(6) + RAND(6) + dm2) : 0;
  }

  if(biodam > 0) {
    fprintf(fr, " Biosphere damage has occured:\n");
    newsys.port = 5;
    fprintf(fr, "  Starport %c destroyed. ", *port_types[oldsys.port]);
    if(biodam > 0 && biodam < 4)
      fprintf(fr, "Temporary temperature decrease.\n");
    if(biodam > 3 && biodam < 6) {
      dm2 = RAND(6) + 6;
      fprintf(fr, "Permanent temperature decrease of %dC\n", dm2);
    }
    if(biodam > 5 && biodam < 11) {
      if(newsys.atm == 5) {
	newsys.atm = 4;
        fprintf(fr, "Atmosphere accuires a taint. ");
      }
      if(newsys.atm == 6) {
	newsys.atm = 7;
        fprintf(fr, "Atmosphere accuires a taint. ");
      }
      if(newsys.atm == 8) {
	newsys.atm = 9;
        fprintf(fr, "Atmosphere accuires a taint. ");
      }
      if(biodam > 8) {
	fprintf(fr, "Widespread destruction.\n");
	popred = 1;
	techred = 3;
      } else {
	fprintf(fr, "\n");
      }
    }
    if(biodam > 10 && biodam < 13) {
      if(newsys.atm > 0) {
	fprintf(fr, "Atmosphere becomes poisoned. ");
	newsys.atm = 12;
      }
      fprintf(fr, "Extensive devastation.\n");
      popred = 2;
      techred = 6;
    }
    if(biodam > 12) {
      if(newsys.atm > 0) {
	fprintf(fr, "Atmosphere becomes poisoned. ");
	newsys.atm = 12;
      }
      fprintf(fr, "World depopulated (%de%d).\n", newsys.popmul, newsys.pop); 
      newsys.pop = 0; 
      newsys.popmul = 0; 
      newsys.gov = 0;
      newsys.law = 0;
      newsys.tech = 0;
      annihilated = TRUE;
    }
  }

  /* Starport Value Decrease */

  dm = RAND(6);
  dm2 = 0;

  if(oldsys.port < 4) {
    
    dm += svdm[oldsys.area-1][oldsys.port];
    dm += svdm[oldsys.war+3][oldsys.port];
    dm += (int)(isolated(i,j) * (float)svdm[8][oldsys.port]);
    dm += svdm[popeff[0][oldsys.pop]][oldsys.port];
    dm += svdm[tleff[0][oldsys.tech]][oldsys.port];

    dm2 = min(newsys.port + change(dm),5);

    if(dm2 > newsys.port) {
      newsys.port = dm2;
      fprintf(fr, " Starport reduced %c -> %c (-%d).",
	*port_types[oldsys.port], *port_types[newsys.port], change(dm));
      lf = TRUE;
    }

  }
  if(oldsys.base > 0 && ((newsys.port - oldsys.port) >= 2 || 
     newsys.port == 5 || newsys.area > 3 || (RAND(6) > 3))) {
    newsys.base = 0;
    fprintf(fr, " Bases destroyed (%c).",*base_types[oldsys.base]); 
    lf = TRUE;
  }

  /* Recession in planetary economies */

  dm = RAND(6);
  dm2 = 0;

  dm += portdm[0][newsys.port];
  dm += areadm[0][newsys.area];
  dm += atmeff[1][newsys.atm];
  dm += hydeff[1][newsys.hyd];
  dm += popeff[1][newsys.pop];
  dm += goveff[1][newsys.gov];
  dm += tleff[1][newsys.tech];
  dm += fluid(newsys.atm,newsys.hyd) ? 1 : 0;

  dm2 = change(dm) + techred;
  techred = dm2;

  newsys.tech = max(newsys.tech - dm2, 0);

  if(!annihilated && newsys.tech < oldsys.tech) {
    fprintf(fr, " TL reduced %c -> %c (-%d).", 
      n2h(oldsys.tech), n2h(newsys.tech), dm2);
    lf = TRUE;
  }

  /* Newline perchanse? */

  if(lf) fprintf(fr, "\n");
  lf = FALSE;

  /* Xenophobia and Isolationism */
  
  if(!annihilated) {

    /* Xeno */

    dm = RAND(6)/2;
    dm2 = 0;

    dm += newsys.port - oldsys.port;
    dm += biodam ? 6 : 0;
    dm += areadm[1][newsys.area];
    dm += atmeff[2][newsys.atm];
    dm += hydeff[2][newsys.hyd];
    dm += popeff[2][newsys.pop];
    dm += goveff[2][newsys.gov];
    dm += fluid(newsys.atm,newsys.hyd) ? 1 : 0;

    xeno = change(dm);


    if(xeno) {

      /* Isol */ 

      dm = 0;

      dm += atmeff[3][newsys.atm];
      dm += fluid(newsys.atm,newsys.hyd) ? 0 : hydeff[3][newsys.hyd];
      dm += tleff[2][newsys.tech];
      dm += portdm[1][newsys.port];

      isol = (dm + RAND(6) + RAND(6)) > 10;

    }

    /* Doomed or Failing? */

    dm = 0;
    dm2 = 0;

    if(newsys.tech < 9) {
      dm += dfdm[atmeff[4][newsys.atm]][newsys.tech];
      dm += dfdm[hydeff[4][newsys.hyd]][newsys.tech];
      dm += fluid(newsys.atm,newsys.hyd) ? dfdm[9][newsys.tech] : 0;
    }
    dm += popeff2[0][newsys.pop];
    if(newsys.war > 1) {
      dm += popeff2[newsys.war-1][newsys.pop];
    }
    envdm = change(dm);
    dm += RAND(6)/2;
    dm2 = change(dm) + popred;
    popred = dm2;

    if(dm2) {
      while((newsys.popmul = newsys.popmul - dm2) <= 0) {
	newsys.pop = newsys.pop - 1;
	dm2 = 0 - newsys.popmul;
	newsys.popmul = 9;
      }

      if(newsys.pop <= 0) {
        newsys.pop = 0;
	newsys.popmul = 0;
        newsys.gov = 0;
        newsys.law = 0;
        newsys.tech = 0;
	annihilated = TRUE;
	newsys.base = 0;
      }

      if(envdm>4) {
	fprintf(fr, " Doomed world.");
      } else {
	if(envdm>1) {
	  fprintf(fr, " Failing world.");
        }
      }
      fprintf(fr, " Population reduced %de%d -> %de%d (-%d).\n",
	oldsys.popmul, oldsys.pop, newsys.popmul, newsys.pop, popred);
    }
  }

  /* Government Change */

  if(!annihilated && (popred || techred)) {
    dm = RAND(6);
    dm2 = 0;

    dm += max(oldsys.tech - newsys.tech - 2, 0);
    dm += max(popred - 1, 0);

    dm2 = min(initgov[oldsys.gov] + change(dm), 13);

    if(dm2 == 7 || dm2 == 11 || dm2 == 12) {
      if(dm2 == 7) dm2 = RAND(6) > 3 ? 7 : 14;
      if(dm2 == 11) dm2 = RAND(6) > 3 ? 11 : 15;
      if(dm2 == 12) dm2 = RAND(6) > 3 ? 12 : 16;
    }

    newsys.gov = thegov[dm2];

    if(thegov[dm2] == 7) balk = TRUE;

    if(change(dm)) {
      fprintf(fr, " Government: %c [%s] -> %c [%s].\n",
	n2h(oldsys.gov), oldgov[oldsys.gov], n2h(newsys.gov),
	newgov[dm2]);
    }

    /* Law Level Change */

    newsys.law = max(min(RAND(6)-7+newsys.gov,20),0);

    if(newsys.law != oldsys.law || xeno) {
      if(!xeno) {
	fprintf(fr, " Lawlevel change: %c -> %c (%+d).\n", n2h(oldsys.law),
	  n2h(newsys.law), newsys.law - oldsys.law);
      } else {
        xeno2 = min(newsys.law + xeno, 20);
        fprintf(fr, " Law level change: %c -> %c/%c (%+d) [Xenophobia: +%d].", 
	  n2h(oldsys.law), n2h(newsys.law), n2h(xeno2),
	  newsys.law - oldsys.law, xeno2 - newsys.law);
        if(isol) {
  	  fprintf(fr, " Isolationist.\n");
        } else {
	  fprintf(fr, "\n");
        }
      }
    }
  }

  /* Virus Time! */

  fprintf(fr, " Virus effects:");

  /* Max Sustainable Population */

  dm = 10;
  dm2 = RAND(9);

  dm += sizeff[0][newsys.size];
  dm += atmeff[5][newsys.atm];
  dm += hydeff[5][newsys.hyd];

  if(newsys.atm > 9 || newsys.atm < 4) dm = 0;

  if(newsys.pop > dm) {
    fprintf(fr, " Catastrophic dieoff:");
    if(dm == 0) {
      fprintf(fr, " World depopulated (%de%d).", newsys.popmul, newsys.pop);
      newsys.pop = 0;
      newsys.popmul = 0;
      newsys.port = 5;
      newsys.gov = 0;
      newsys.law = 0;
      newsys.tech = 0;
      newsys.base = 0;
      annihilated = TRUE;
    } else {
      fprintf(fr, " Population rapidly falls %de%d -> %de%d.",
	newsys.popmul, newsys.pop, dm2, dm);
      newsys.pop = dm;
      newsys.popmul = dm2;
    }
  }

  maxpop = dm;
  maxpopmul = max(newsys.popmul, dm2);

  fprintf(fr, "\n");

  /* Tech Level Decline */

  if(!annihilated) {
    dm = 0;
    dm2 = 0;

    if(oldsys.tech < 9) dm = max(RAND(6)-3,0);
    if(oldsys.tech > 8 && oldsys.tech < 11) dm = RAND(6);
    if(oldsys.tech > 10 && oldsys.tech < 14) dm = RAND(6) + RAND(6);
    if(oldsys.tech > 13) dm = RAND(6) + RAND(6) + RAND(6);

    dm2 = max(newsys.tech - dm, 0);

    lf = FALSE;

    if(dm) {
      fprintf(fr, "  Techlevel falls %c -> %c (-%d).", 
        n2h(newsys.tech), n2h(dm2), newsys.tech - dm2);
      techred = newsys.tech - dm2;
      newsys.tech = dm2;
      lf = TRUE;
    }

    /* Actual population */

    dm2 = (int)((float)dm / 4.0 + 0.5);
    dm = dm2;
    pop = newsys.pop;
    popmul = newsys.popmul;

    if(dm2) {
      while((popmul = popmul - dm2) <= 0) {
        pop = pop - 1;
        dm2 = 0 - popmul;
        popmul = 9;
      }
      if(pop > 0) {
        fprintf(fr, "  Population reduced %de%d -> %de%d.",
          newsys.popmul, newsys.pop, popmul, pop);
        lf = TRUE;
        newsys.popmul = popmul;
        newsys.pop = pop;
      } else {
       fprintf(fr, "  World depopulated (%de%d).", newsys.popmul, newsys.pop);
        newsys.pop = 0;
        newsys.popmul = 0;
        newsys.port = 5;
        newsys.gov = 0;
        newsys.law = 0;
        newsys.tech = 0;
        newsys.base = 0;
        annihilated = TRUE;
	lf = TRUE;
      }
    }
  }

  if(lf) fprintf(fr, "\n");
  lf = FALSE;

  if(!annihilated && newsys.pop < 6) {
    popmul = popmul/2;
    if(!popmul) {
      popmul = 5;
      pop--;
    }
    fprintf(fr, "  Insufficient population to support itself.");
    if(pop > 0) {
      dm = newsys.tech - 1;
      if(dm >= 0) {
	newsys.tech = newsys.tech - 1;
        fprintf(fr, " TL: %c -> %c.", n2h(newsys.tech +1), n2h(newsys.tech));
        techred++;
      }
      fprintf(fr, " Pop: %de%d -> %de%d.\n", newsys.popmul, newsys.pop,
	popmul, pop);
      newsys.popmul = popmul;
      newsys.pop = pop;
    } else {
      fprintf(fr, " World depopulated (%de%d).\n", newsys.popmul, newsys.pop);
      newsys.pop = 0;
      newsys.popmul = 0;
      newsys.port = 5;
      newsys.gov = 0;
      newsys.law = 0;
      newsys.tech = 0;
      newsys.base = 0;
      annihilated = TRUE;
    } 
  }

  /* Starport */

  if(newsys.port != 5) {
    port = newsys.port;
    dm = RAND(6);
    if(dm > techred) {
      port = min(newsys.port + 1, 5);
    } else {
      if(dm = techred) {
	port = min(newsys.port + 2,5);
      } else {
	port = 5;
      }
    }
    if(newsys.port < port) {
      fprintf(fr, "  Starport reduced %c -> %c.",
 	*port_types[newsys.port], *port_types[port]);
      dm2 = newsys.port;
      newsys.port = port;
      lf = TRUE;
      if(newsys.base > 0 && ((newsys.port - dm2) >= 2 || 
         newsys.port == 5 || (RAND(6) > 2))) {
        newsys.base = 0;
        fprintf(fr, " Bases destroyed (%c).",*base_types[oldsys.base]); 
        lf = TRUE;
      }
    }
    if(lf) fprintf(fr, "\n");
  }
  lf = FALSE;

  /* Balcanization */

  balk = FALSE;

  strcpy(newsys.alle,"--");

  strcpy(s,"");

  if(!annihilated) {
    dm = newsys.size + newsys.pop - newsys.tech;

    if((RAND(6)+RAND(6)) <= dm) {
      balk = TRUE;
      gov = 7;
    }
  }

  /* TED */

  if(!annihilated) {
    if(newsys.pop > 4 && (RAND(10) < techred)) {
      ted = TRUE;
      gov = tedgov[RAND(6) + RAND(6)];
    } else {
      dm = min(max(RAND(6) + RAND(6) - 7 + newsys.pop,0),12);
      if(dm == 6) {
	ted = TRUE;
	gov = tedgov[RAND(6) + RAND(6)];
      }
      gov = wildgov[dm];
    }
  }

  if(balk) {
    newsys.gov = 7;
    fprintf(fr, "  Balkanized, most common ");
    if(!ted) {
      fprintf(fr, "%c [%s].\n", n2h(gov), oldgov[gov]);
      strcpy(s,"B:");
      s[2] = n2h(gov);
      s[3] = (char)NULL;
      strcat(s," ");
    } else {
      fprintf(fr, "TED: %c [%s].\n", n2h(gov), oldgov[gov]);
      strcpy(s,"B:");
      s[2] = n2h(gov);
      s[3] = (char)NULL;
      strcat(s," ");
      strcpy(newsys.alle, "Td");
    }
  } else {
    if(ted) {
      fprintf(fr, "  TED:  %c [%s].\n", n2h(gov), oldgov[gov]);
      newsys.gov = (int)gov;
      strcpy(newsys.alle, "Td");
    } 
  }

  /* LAW */

  if(!annihilated) {
    dm = 0;
    dm = (ted) ? RAND(6) + RAND(6) + 3 : RAND(6) + RAND(6) - 7 + gov;
    dm2 = min(max(dm,0),20);

    if(newsys.law != dm2) {
      if(!xeno) {
	fprintf(fr, "  Lawlevel changes: %c -> %c (%+d).\n", n2h(newsys.law),
        n2h(dm2), dm2 - newsys.law);
	newsys.law = dm2;
      } else {
        dm = min(dm2 + xeno, 20);
       fprintf(fr, "  Lawlevel change: %c/%c -> %c/%c (%+d) [Xenophobia: +%d].",
        n2h(newsys.law), n2h(xeno2), n2h(dm2), n2h(dm),
        dm2 - newsys.law, dm - dm2);
	newsys.law = dm2;
        if(isol) {
          fprintf(fr, " Isolationist.\n");
        } else {
          fprintf(fr, "\n");
        }
      }
    }
  }

  /* Population Regrowth */

  if(!annihilated && newsys.pop > 5 && envdm < 5) {
    growth = grow[newsys.tech];
    if(envdm > 1) growth = 1;
    if((newsys.atm == 6 || newsys.atm == 8) &&
      (newsys.pop > 5 && newsys.pop < 9) &&
      (newsys.gov > 3 && newsys.gov < 10) && !ted) growth++;

    dm2 = (int)((float)newsys.popmul * rate[growth]);
    dm2 = dm2 - newsys.popmul;
    dm = dm2;

    if(dm2) {
      popmul = (newsys.popmul+dm2)%9;
      pop = newsys.pop + (newsys.popmul + dm2)/9;
      if(pop>maxpop) {
	pop = maxpop;
	popmul = maxpopmul;
	hit = TRUE;
      } else {
	if(pop == maxpop && popmul >= maxpopmul) {
	  popmul = maxpopmul;
	  hit = TRUE;
        }
      }
      if(popmul == newsys.popmul && pop == newsys.pop) {
	fprintf(fr, "Population growth held back by environment.\n");
      } else {
        fprintf(fr,
	  "  Population grows %de%d -> %de%d", newsys.popmul, newsys.pop,
	  popmul, pop);
        if(hit) fprintf(fr, ", and hits the maximum supportable level");
        fprintf(fr, ".\n");
	newsys.popmul = popmul;
	newsys.pop = pop;
      }
    }
  }

  /* New Tradeclasses */

  strcpy(e,"");
 
  if(strstr(oldsys.trade, "In")) industrial = TRUE;

  sscanf(oldsys.trade, "D%d", &droyne);
  sscanf(oldsys.trade, "C%d", &chirpers);
  
  if(droyne && pop) {
    strcpy(e, "D");
    e[1] = n2h(droyne);
    e[2] = (char)NULL;
    strcat(e, " ");
  }

  if(chirpers && pop) {
    strcpy(newsys.trade, "C");
    e[1] = n2h(chirpers);
    e[2] = (char)NULL;
    strcat(newsys.trade, " ");
  }

  strcat(e, s);

  if(newsys.pop<=4) strcat(e, "Lo ");
  if(newsys.pop>=9) strcat(e, "Hi ");
  if(newsys.atm<=3 && newsys.hyd <=3 && newsys.pop >= 6)
    strcat(e, "Na ");
  if(newsys.atm > 3 && newsys.atm < 10 && newsys.hyd>3 && newsys.hyd<9 &&
     newsys.pop > 4 && newsys.pop < 8) strcat(e, "Ag ");
  if(newsys.pop<=6) strcat(e, "Ni ");
  if((newsys.atm<3 || newsys.atm == 4 || newsys.atm == 7 || newsys.atm == 9) &&
     newsys.pop > 8 && industrial) strcat(e, "In ");
  if(newsys.atm>1 && newsys.atm <6 && newsys.hyd < 4)
    strcat(e, "Po ");
  if((newsys.atm == 6 || newsys.atm == 8) &&
    (newsys.pop > 5 && newsys.pop < 9) &&
    (newsys.gov > 3 && newsys.gov < 10)) strcat(e, "Ri ");
  if(newsys.atm>=2 && newsys.hyd == 0) strcat(e, "De ");
  if(newsys.hyd==10) strcat(e, "Wa ");
  if(newsys.size == 0 && newsys.atm == 0 && newsys.hyd == 0) {
    strcat(e, "As ");
  } else {
    if(newsys.atm == 0) strcat(e, "Va ");
  }
  if(newsys.atm<=1 && newsys.hyd > 0) strcat(e, "Ic ");
  if(newsys.atm>9 && oldsys.atm >9 && newsys.hyd > 0)
    strcat(e, "Fl ");
  if(newsys.pop == 0) strcat(e, "Ba ");

  rem_trail_sp(e);

  sprintf(newsys.trade, "%-.15s", e);

  /* New Zones */

  newsys.zone = 0;

  /*
  if(ted || xeno) newsys.zone = 1;
  if(!newsys.pop) newsys.zone = 2;
  */

  /* Print new UWP */

  fprintf(fr, "  New UWP: %c%c%c%c%c%c%c-%c%s\n", 
    *port_types[newsys.port],
    n2h(newsys.size),
    n2h(newsys.atm),
    n2h(newsys.hyd),
    n2h(newsys.pop),
    n2h(newsys.gov),
    n2h(newsys.law),
    n2h(newsys.tech),
    zone_types[newsys.zone]);

  /* New entry comming up */

  fprintf(fr, "\n");

  /* And save the old one */
      
  newsector[i][j] = newsys;
}

static int min(a,b)
int a,b;
{
  if(a<b) return a;
  return b;
}

static int max(a,b)
int a,b;
{
  if(a<b) return b;
  return a;
}

static int change(x)
int x;
{
  if (x<=2) return 0;
  return (int)((((float)x-2.0)/2.0)-0.5);
}

static float isolated(i,j)
int i,j;
{
  int x;
  int n = 0;
  int m = 0;
  for(x = 0;x < 18;x++) {
    if((i+isol[0][x] <= 31 && i+isol[0][x] >=0) &&
       (j+isol[1][x] <= 39 && j+isol[1][x] >=0)) {
      n += newsector[i][j].presence;
    } else m++;
  }
  if(n > 0) return 0.0;
  if(m > 0) return 0.5;
  return 1.0;
}

static int fluid(i,j)
int i,j;
{
  if(i >= 10 && j >= 1) return 1;
  return 0;
}
