/*
 * readsector.c
 *
 * Handles the reading of the sector data, decodes it and places it
 * in the global array of structures called sector
 * 
 * Copyright (c) 1993 Bertil K Jonell 
 *
 */

#include<stdio.h>
#include<strings.h>
#include<ctype.h>
#include"include.h"

extern struct system sector[][40];

void read_sector(file)
char file[];
{
  int c, i;
  char s[MAXSTR];
  FILE *f1, *fopen();
  static void store_system();

  if ((f1 = fopen(file, "r")) == NULL) {
    fprintf(stderr, "I can't open %s\n", file);
    exit(1);
  }

  fprintf(stderr, "%s\n", file);
  
  while (fgets(s, sizeof(s), f1) != NULL) {
    store_system(s);
  }
  fclose(f1);
}

static void store_system(s)
char s[];
{
  char name[15];
  int x, y;
  char port, size, atm, hyd, pop, gov, law, tech;
  char base;
  char trade[16];
  char zone;
  int popmul, plan, gas;
  char alle[3];
  char stars[17];
  int test;
  static void rem_trail_sp(), rem_trail();
  static int htoi(), port2num(), base2num(), zone2num();

  strcpy(stars, "                ");
  test = sscanf(s,
   "%14c%2d%2d%*c%c%c%c%c%c%c%c-%c%*2c%c%*c%15c%*c%c%*2c%1d%1d%1d%*c%2c%*c%16c",
    name, &x, &y, &port, &size, &atm, &hyd, &pop, &gov, &law, &tech, &base, 
    trade, &zone, &popmul, &plan, &gas, alle, stars);
  name[14] = (char)NULL;
  trade[15] = (char)NULL;
  alle[2] = (char)NULL;
  stars[16] = (char)NULL;

  if(test != 19) {
    fprintf(stderr, "Error reading sector file: %d %d\n", x, y);
    return;
  }

  rem_trail_sp(name);
  rem_trail_sp(trade);
  alle[2] = (char)NULL;
  rem_trail_sp(alle);
  rem_trail(stars);
  
  strcpy(sector[x-1][y-1].name, name);
  strcpy(sector[x-1][y-1].trade, trade);
  strcpy(sector[x-1][y-1].alle, alle);
  strcpy(sector[x-1][y-1].stars, stars);

  sector[x-1][y-1].presence = TRUE;
  sector[x-1][y-1].popmul = popmul;
  sector[x-1][y-1].plan = plan;
  sector[x-1][y-1].gas = gas;

  sector[x-1][y-1].size = htoi(size);
  sector[x-1][y-1].atm = htoi(atm);
  sector[x-1][y-1].hyd = htoi(hyd);
  sector[x-1][y-1].pop = htoi(pop);
  sector[x-1][y-1].gov = htoi(gov);
  sector[x-1][y-1].law = htoi(law);
  sector[x-1][y-1].tech = htoi(tech);

  sector[x-1][y-1].port = port2num(port);
  sector[x-1][y-1].base = base2num(base);
  sector[x-1][y-1].zone = zone2num(zone);
}


static int port2num(c)
char c;
{
  if(c == 'A') return 0;
  if(c == 'B') return 1;
  if(c == 'C') return 2;
  if(c == 'D') return 3;
  if(c == 'E') return 4;
  if(c == 'X') return 5;
  return 5;
}


static int base2num(c)
char c;
{
  if(c == 'S') return 1;
  if(c == 'W') return 2;
  if(c == 'N') return 4;
  if(c == 'D') return 8;
  if(c == 'M') return 16;
  if(c == 'A') return 5;
  if(c == 'B') return 6;
  return 0;
}


static int zone2num(c)
char c;
{
  if(c == 'A') return 1;
  if(c == 'R') return 2;
  return 0;
}


static int htoi(s)
char s;
{
  if (isdigit(s)) return atoi(&s);
  return ((int)s - 55);
}


void rem_trail_sp(s)
char *s;
{
    int i = strlen(s) - 1;

    while (i >= 0 && s[i] == ' ') {
        s[i--] = '\0';
    }
}


static void rem_trail(s)
char *s;
{
    int i = strlen(s) - 1;
    while (i >= 0 && (s[i] == ' ' || s[i] == '\n' || !isascii(s[i]))) {
      s[i--] = '\0';
    }
}
